@@ -137,13 +137,13 @@ namespace H265 {
         uint8_t initial_cpb_removal_delay_length_minus1;
         uint8_t au_cpb_removal_delay_length_minus1;
         uint8_t dpb_output_delay_length_minus1;
-        bool fixed_pic_rate_general_flag[6];
-        bool fixed_pic_rate_within_cvs_flag[6];
-        uint16_t elemental_duration_in_tc_minus1[6]; //[0, 2047]
-        bool low_delay_hrd_flag[6];
-        uint8_t cpb_cnt_minus1[6]; //[0, 31]
+        bool fixed_pic_rate_general_flag[MAXSUBLAYERS];
+        bool fixed_pic_rate_within_cvs_flag[MAXSUBLAYERS];
+        uint16_t elemental_duration_in_tc_minus1[MAXSUBLAYERS]; //[0, 2047]
+        bool low_delay_hrd_flag[MAXSUBLAYERS];
+        uint8_t cpb_cnt_minus1[MAXSUBLAYERS]; //[0, 31]
 
-        SubLayerHRDParameters sublayer_hrd_params[6];
+        SubLayerHRDParameters sublayer_hrd_params[MAXSUBLAYERS];
     };
 
     struct ShortTermRefPicSet {
@@ -217,9 +217,15 @@ namespace H265 {
         uint8_t scalingList4x4[6][16];
         uint8_t scalingList8x8[6][64];
         uint8_t scalingList16x16[6][64];
-        uint8_t scalingList32x32[2][64];
+        // According to spec "7.3.4 Scaling list data syntax",
+        // we just use scalingList32x32[0] and scalingList32x32[3];
+        // as spec "7.4.5 Scaling list data semantics",
+        // matrixId as the index of scalingList32x32 can be equal to 3;
+        // so we'd better define scalingList32x32[6][] other than scalingList32x32[2][].
+        uint8_t scalingList32x32[6][64];
         uint8_t scalingListDC16x16[6];
-        uint8_t scalingListDC32x32[2];
+        // scalingListDC32x32 is similar to scalingList32x32.
+        uint8_t scalingListDC32x32[6];
     };
 
     struct RefPicListModification {
@@ -302,6 +308,8 @@ namespace H265 {
         int32_t width;
         int32_t height;
         //cropped frame
+        uint32_t croppedLeft;
+        uint32_t croppedTop;
         uint32_t croppedWidth;
         uint32_t croppedHeight;
         uint8_t bit_depth_luma_minus8; //[0, 8]
