@@ -43,6 +43,8 @@ using ::std::ref;
 
 namespace YamiMediaCodec {
 
+#define JPEG_SURFACE_NUM 2
+
 struct Slice {
     Slice() : data(NULL), start(0) , length(0) { }
 
@@ -77,29 +79,18 @@ public:
         if (!data || !size)
             return YAMI_SUCCESS;
 
-        /*
-         * Reset the parser if we have a new data pointer; this is common for
-         * MJPEG. If the data pointer is the same, then the assumption is that
-         * we are continuing after previously suspending due to an SOF
-         * YAMI_DECODE_FORMAT_CHANGE.
-         */
-        if (m_slice.data != data)
-            m_parser.reset();
-
-        if (!m_parser) { /* First call or new data */
-            Parser::Callback defaultCallback =
-                bind(&Impl::onMarker, ref(*this));
-            Parser::Callback sofCallback =
-                bind(&Impl::onStartOfFrame, ref(*this));
-            m_slice.data = data;
-            m_parser.reset(new Parser(data, size));
-            m_parser->registerCallback(M_SOI, defaultCallback);
-            m_parser->registerCallback(M_EOI, defaultCallback);
-            m_parser->registerCallback(M_SOS, defaultCallback);
-            m_parser->registerCallback(M_DHT, defaultCallback);
-            m_parser->registerCallback(M_DQT, defaultCallback);
-            m_parser->registerStartOfFrameCallback(sofCallback);
-        }
+        Parser::Callback defaultCallback =
+            bind(&Impl::onMarker, ref(*this));
+        Parser::Callback sofCallback =
+            bind(&Impl::onStartOfFrame, ref(*this));
+        m_slice.data = data;
+        m_parser.reset(new Parser(data, size));
+        m_parser->registerCallback(M_SOI, defaultCallback);
+        m_parser->registerCallback(M_EOI, defaultCallback);
+        m_parser->registerCallback(M_SOS, defaultCallback);
+        m_parser->registerCallback(M_DHT, defaultCallback);
+        m_parser->registerCallback(M_DQT, defaultCallback);
+        m_parser->registerStartOfFrameCallback(sofCallback);
 
         if (!m_parser->parse())
             m_decodeStatus = YAMI_FAIL;
@@ -144,8 +135,13 @@ private:
                 + m_parser->current().length;
             break;
         case M_EOI:
-            m_slice.length = m_parser->current().position - m_slice.start;
-            m_decodeStatus = m_finishHandler();
+            if (m_slice.start >= m_parser->current().position) {
+                // we can't have zero or negative length
+                m_decodeStatus = YAMI_FAIL;
+            } else {
+                m_slice.length = m_parser->current().position - m_slice.start;
+                m_decodeStatus = m_finishHandler();
+            }
             break;
         case M_DQT:
             m_quantizationTables = m_parser->quantTables();
@@ -343,8 +339,8 @@ YamiStatus VaapiDecoderJPEG::loadHuffmanTables()
 
 YamiStatus VaapiDecoderJPEG::decode(VideoDecodeBuffer* buffer)
 {
-    if (!buffer)
-        return YAMI_FAIL;
+    if (!buffer || !buffer->data)
+        return YAMI_SUCCESS;
 
     m_currentPTS = buffer->timeStamp;
 
@@ -366,8 +362,11 @@ YamiStatus VaapiDecoderJPEG::decode(VideoDecodeBuffer* buffer)
 //get frame fourcc, return 0 for unsupport format
 static uint32_t getFourcc(const FrameHeader::Shared& frame)
 {
+    if (frame->components.size() == 1) // monochrome image
+        RETURN_FORMAT(YAMI_FOURCC_Y800);
+
     if (frame->components.size() != 3) {
-        ERROR("unsupported compoent size %d", (int)frame->components.size());
+        ERROR("unsupported component size %d", (int)frame->components.size());
         return 0;
     }
     int h1 = frame->components[0]->hSampleFactor;
@@ -400,44 +399,7 @@ YamiStatus VaapiDecoderJPEG::start(VideoConfigBuffer* buffer)
 {
     DEBUG("%s", __func__);
 
-    m_configBuffer = *buffer;
-    m_configBuffer.surfaceNumber = 2;
-    m_configBuffer.profile = VAProfileJPEGBaseline;
-
-    /* We can't start until decoding has started */
-    if (!m_impl.get())
-        return YAMI_SUCCESS;
-
-    const FrameHeader::Shared frame = m_impl->frameHeader();
-
-    /*
-     * We don't expect the user to call start() after a failed decode() attempt.
-     * But since the user can, we must check if we have a valid frame header
-     * before we can use it.  That is, m_parser could be initialized but the
-     * frame header might not be.
-     */
-    if (!frame)
-        return YAMI_FAIL;
-
-    if (!frame->isBaseline) {
-        ERROR("Unsupported JPEG profile. Only JPEG Baseline is supported.");
-        return YAMI_FAIL;
-    }
-
-    m_configBuffer.width = frame->imageWidth;
-    m_configBuffer.height = frame->imageHeight;
-    m_configBuffer.surfaceWidth = frame->imageWidth;
-    m_configBuffer.surfaceHeight = frame->imageHeight;
-    m_configBuffer.fourcc = getFourcc(frame);
-    if (!m_configBuffer.fourcc) {
-        return YAMI_UNSUPPORTED;
-    }
-
-    /* Now we can actually start */
-    if (VaapiDecoderBase::start(&m_configBuffer) != YAMI_SUCCESS)
-        return YAMI_FAIL;
-
-    return YAMI_DECODE_FORMAT_CHANGE;
+    return YAMI_SUCCESS;
 }
 
 YamiStatus VaapiDecoderJPEG::finish()
@@ -452,15 +414,21 @@ YamiStatus VaapiDecoderJPEG::finish()
         return YAMI_FAIL;
     }
 
-    m_picture = createPicture(m_currentPTS);
-    if (!m_picture) {
+    YamiStatus status;
+    status = ensureContext();
+    if (status != YAMI_SUCCESS) {
+        return status;
+    }
+    status = createPicture(m_picture, m_currentPTS);
+    if (status != YAMI_SUCCESS) {
         ERROR("Could not create a VAAPI picture.");
-        return YAMI_FAIL;
+        return status;
     }
 
     m_picture->m_timeStamp = m_currentPTS;
-
-    YamiStatus status;
+    SurfacePtr surf;
+    surf = m_picture->getSurface();
+    surf->setCrop(0, 0, m_videoFormatInfo.width, m_videoFormatInfo.height);
 
     status = fillSliceParam();
     if (status !=  YAMI_SUCCESS) {
@@ -508,4 +476,21 @@ YamiStatus VaapiDecoderJPEG::reset(VideoConfigBuffer* buffer)
     return VaapiDecoderBase::reset(buffer);
 }
 
+YamiStatus VaapiDecoderJPEG::ensureContext()
+{
+    const FrameHeader::Shared frame = m_impl->frameHeader();
+    if (!frame->isBaseline) {
+        ERROR("Unsupported JPEG profile. Only JPEG Baseline is supported.");
+        return YAMI_FAIL;
+    }
+
+    if (!getFourcc(frame)) {
+        return YAMI_UNSUPPORTED;
+    }
+    if (setFormat(frame->imageWidth, frame->imageHeight, frame->imageWidth,
+            frame->imageHeight, JPEG_SURFACE_NUM, getFourcc(frame))) {
+        return YAMI_DECODE_FORMAT_CHANGE;
+    }
+    return ensureProfile(VAProfileJPEGBaseline);
+}
 }
