@@ -56,18 +56,17 @@ VaapiDecoderBase::~VaapiDecoderBase()
     stop();
 }
 
-PicturePtr VaapiDecoderBase::createPicture(int64_t timeStamp /* , VaapiPictureStructure structure = VAAPI_PICTURE_STRUCTURE_FRAME */)
+YamiStatus VaapiDecoderBase::createPicture(PicturePtr& picture, int64_t timeStamp /* , VaapiPictureStructure structure = VAAPI_PICTURE_STRUCTURE_FRAME */)
 {
-    PicturePtr picture;
     /*accquire one surface from m_surfacePool in base decoder  */
     SurfacePtr surface = createSurface();
     if (!surface) {
         DEBUG("create surface failed");
-        return picture;
+        return YAMI_DECODE_NO_SURFACE;
     }
 
     picture.reset(new VaapiDecPicture(m_context, surface, timeStamp));
-    return picture;
+    return YAMI_SUCCESS;
 }
 
 YamiStatus VaapiDecoderBase::start(VideoConfigBuffer* buffer)
@@ -165,51 +164,18 @@ void VaapiDecoderBase::flush(void)
 {
 
     INFO("base: flush()");
-    if (m_surfacePool) {
-        m_surfacePool->flush();
-    }
+    m_output.clear();
 
     m_currentPTS = INVALID_PTS;
 }
 
-struct BufferRecycler
-{
-    BufferRecycler(const DecSurfacePoolPtr&  pool, VideoRenderBuffer* buffer)
-        :m_pool(pool), m_buffer(buffer)
-    {
-    }
-    void operator()(VideoFrame* frame)
-    {
-        m_pool->recycle(m_buffer);
-        delete frame;
-    }
-
-private:
-    DecSurfacePoolPtr  m_pool;
-    VideoRenderBuffer* m_buffer;
-};
-
 SharedPtr<VideoFrame> VaapiDecoderBase::getOutput()
 {
     SharedPtr<VideoFrame> frame;
-    DecSurfacePoolPtr pool = m_surfacePool;
-    if (!pool)
+    if (m_output.empty())
         return frame;
-    VideoRenderBuffer *buffer = pool->getOutput();
-    if (buffer) {
-        frame.reset(new VideoFrame, BufferRecycler(pool, buffer));
-        memset(frame.get(), 0, sizeof(VideoFrame));
-        frame->surface = (intptr_t)buffer->surface;
-        frame->timeStamp = buffer->timeStamp;
-        frame->crop = buffer->crop;
-        frame->fourcc = buffer->fourcc;
-#ifdef __ENABLE_DEBUG__
-        if (m_dumpSurface && access("/tmp/yami/dumpon", F_OK) == 0) {
-            DEBUG("dumpSurface display: %p, surface: 0x%x", buffer->display, buffer->surface);
-            dumpSurface(buffer->display, buffer->surface);
-        }
-#endif
-    }
+    frame = m_output.front();
+    m_output.pop_front();
     return frame;
 }
 
@@ -247,12 +213,12 @@ VaapiDecoderBase::setupVA(uint32_t numSurface, VAProfile profile)
     VAConfigAttrib attrib;
     attrib.type = VAConfigAttribRTFormat;
     attrib.value = VA_RT_FORMAT_YUV420;
+    ConfigPtr config;
 
-
-    ConfigPtr config = VaapiConfig::create(m_display, profile, VAEntrypointVLD,&attrib, 1);
-    if (!config) {
+    YamiStatus status = VaapiConfig::create(m_display, profile, VAEntrypointVLD, &attrib, 1, config);
+    if (YAMI_SUCCESS != status) {
         ERROR("failed to create config");
-        return YAMI_FAIL;
+        return status;
     }
 
     if (!m_externalAllocator) {
@@ -263,7 +229,7 @@ VaapiDecoderBase::setupVA(uint32_t numSurface, VAProfile profile)
     }
 
     m_configBuffer.surfaceNumber = numSurface;
-    m_surfacePool = VaapiDecSurfacePool::create(m_display, &m_configBuffer, m_allocator);
+    m_surfacePool = VaapiDecSurfacePool::create(&m_configBuffer, m_allocator);
     DEBUG("surface pool is created");
     if (!m_surfacePool)
         return YAMI_FAIL;
@@ -289,9 +255,109 @@ VaapiDecoderBase::setupVA(uint32_t numSurface, VAProfile profile)
     return YAMI_SUCCESS;
 }
 
+bool VaapiDecoderBase::createAllocator()
+{
+    if (m_allocator)
+        return true;
+    m_display = VaapiDisplay::create(m_externalDisplay);
+    if (!m_display) {
+        ERROR("failed to create display");
+        return false;
+    }
+
+    if (!m_externalAllocator) {
+        //use internal allocator
+        m_allocator.reset(new VaapiSurfaceAllocator(m_display->getID()), unrefAllocator);
+    }
+    else {
+        m_allocator = m_externalAllocator;
+    }
+    if (!m_allocator) {
+        m_display.reset();
+        ERROR("failed to create allocator");
+        return false;
+    }
+    return true;
+}
+
+bool VaapiDecoderBase::isSurfaceGeometryChanged() const
+{
+    return m_config.width < m_videoFormatInfo.surfaceWidth
+        || m_config.height < m_videoFormatInfo.surfaceHeight
+        || m_config.surfaceNumber != m_videoFormatInfo.surfaceNumber
+        || m_config.fourcc != m_videoFormatInfo.fourcc;
+}
+
+YamiStatus VaapiDecoderBase::ensureSurfacePool()
+{
+
+    if (!isSurfaceGeometryChanged())
+        return YAMI_SUCCESS;
+    VideoFormatInfo& info = m_videoFormatInfo;
+    m_config.width = info.surfaceWidth;
+    m_config.height = info.surfaceHeight;
+    m_config.surfaceNumber = info.surfaceNumber;
+    m_config.fourcc = info.fourcc;
+
+    if (!createAllocator())
+        return YAMI_FAIL;
+
+    m_surfacePool = VaapiDecSurfacePool::create(&m_config, m_allocator);
+    if (!m_surfacePool)
+        return YAMI_FAIL;
+    DEBUG("surface pool is created");
+
+    return YAMI_SUCCESS;
+}
+
+YamiStatus VaapiDecoderBase::ensureProfile(VAProfile profile)
+{
+    YamiStatus status;
+    status = ensureSurfacePool();
+    if (status != YAMI_SUCCESS)
+        return status;
+
+    if (!m_display || !m_surfacePool) {
+        ERROR("bug: no display or surface pool");
+        return YAMI_FAIL;
+    }
+    if (m_config.profile == profile)
+        return YAMI_SUCCESS;
+
+    m_config.profile = profile;
+    VAConfigAttrib attrib;
+    attrib.type = VAConfigAttribRTFormat;
+    attrib.value = VA_RT_FORMAT_YUV420;
+    ConfigPtr config;
+
+    status = VaapiConfig::create(m_display, profile, VAEntrypointVLD, &attrib, 1, config);
+    if (YAMI_SUCCESS != status) {
+        ERROR("failed to create config");
+        return status;
+    }
+
+    std::vector<VASurfaceID> surfaces;
+    m_surfacePool->getSurfaceIDs(surfaces);
+    if (surfaces.empty())
+        return YAMI_FAIL;
+    int size = surfaces.size();
+    m_context = VaapiContext::create(config,
+        m_videoFormatInfo.width,
+        m_videoFormatInfo.height,
+        0, &surfaces[0], size);
+
+    if (!m_context) {
+        ERROR("create context failed");
+        return YAMI_FAIL;
+    }
+    return YAMI_SUCCESS;
+}
+
 YamiStatus VaapiDecoderBase::terminateVA(void)
 {
     INFO("base: terminate VA");
+    m_output.clear();
+    m_config.resetConfig();
     m_surfacePool.reset();
     m_allocator.reset();
     DEBUG("surface pool is reset");
@@ -315,7 +381,6 @@ void VaapiDecoderBase::releaseLock(bool lockable)
     if (!m_surfacePool)
         return;
 
-    m_surfacePool->setWaitable(lockable);
 }
 
 void VaapiDecoderBase::setAllocator(SurfaceAllocator* allocator)
@@ -327,18 +392,28 @@ SurfacePtr VaapiDecoderBase::createSurface()
 {
     SurfacePtr surface;
     if (m_surfacePool) {
-        surface = m_surfacePool->acquireWithWait();
+        surface = m_surfacePool->acquire();
     }
     return surface;
 }
 
+struct VaapiDecoderBase::VideoFrameRecycler {
+    VideoFrameRecycler(const SurfacePtr& surface)
+        : m_surface(surface)
+    {
+    }
+    void operator()(VideoFrame* frame) {}
+private:
+    SurfacePtr m_surface;
+};
+
 YamiStatus VaapiDecoderBase::outputPicture(const PicturePtr& picture)
 {
-    //TODO: reorder poc
-    return m_surfacePool->output(picture->getSurface(),
-               picture->m_timeStamp)
-        ? YAMI_SUCCESS
-        : YAMI_FAIL;
+    SurfacePtr surface = picture->getSurface();
+    SharedPtr<VideoFrame> frame(surface->m_frame.get(), VideoFrameRecycler(surface));
+    frame->timeStamp = picture->m_timeStamp;
+    m_output.push_back(frame);
+    return YAMI_SUCCESS;
 }
 
 VADisplay VaapiDecoderBase::getDisplayID()
@@ -348,4 +423,27 @@ VADisplay VaapiDecoderBase::getDisplayID()
     return m_display->getID();
 }
 
+#define CHECK(f)                        \
+    do {                                \
+        if (m_videoFormatInfo.f != f) { \
+            m_videoFormatInfo.f = f;    \
+            changed = true;             \
+        }                               \
+    } while (0)
+
+bool VaapiDecoderBase::setFormat(uint32_t width, uint32_t height, uint32_t surfaceWidth, uint32_t surfaceHeight,
+    uint32_t surfaceNumber, uint32_t fourcc)
+{
+    bool changed = false;
+    CHECK(width);
+    CHECK(height);
+    CHECK(surfaceWidth);
+    CHECK(surfaceHeight);
+    CHECK(surfaceNumber);
+    CHECK(fourcc);
+    //this not true, but it's only way to let user get format info
+    m_VAStarted = true;
+    return changed;
+}
+
 } //namespace YamiMediaCodec
