@@ -67,6 +67,7 @@ private:
                   bool newStream, bool contextChanged,
                   uint32_t maxDecFrameBuffering);
         bool add(const PicturePtr&);
+        bool outputReadyFrames();
         void initReference(const PicturePtr&, const SliceHeader* const);
         void flush();
 
@@ -74,6 +75,7 @@ private:
         RefSet m_refList1;
 
         PictureList m_pictures;
+        bool m_isLowLatencymode;
 
     private:
         void forEach(ForEachFunction);
@@ -125,6 +127,8 @@ private:
         uint32_t m_maxNumRefFrames;
         uint32_t m_maxDecFrameBuffering;
         YamiParser::H264::DecRefPicMarking m_decRefPicMarking;
+        bool m_isOutputStarted;
+        int32_t m_lastOutputPoc;
     };
 
     YamiStatus decodeNalu(NalUnit*);
@@ -147,7 +151,6 @@ private:
                                    const SliceHeader* const slice,
                                    RefSet& refSet, bool isList0);
     bool isDecodeContextChanged(const SharedPtr<SPS>& sps);
-    bool isResolutionChanged(const SharedPtr<SPS>& sps);
     bool decodeAvcRecordData(uint8_t* buf, int32_t bufSize);
 
     YamiStatus createPicture(const SliceHeader* const,
