@@ -39,6 +39,8 @@ static const uint8_t keyFrameUVModeProbs[3] = { 142, 114, 183 };
 static const uint8_t nonKeyFrameDefaultYModeProbs[4] = { 112, 86, 140, 37 };
 static const uint8_t nonKeyFrameDefaultUVModeProbs[3] = { 162, 101, 204 };
 
+static const uint32_t surfaceNumVP8 = 3;
+
 static YamiStatus getStatus(Vp8ParserResult result)
 {
     YamiStatus status;
@@ -59,9 +61,6 @@ static YamiStatus getStatus(Vp8ParserResult result)
 
 YamiStatus VaapiDecoderVP8::ensureContext()
 {
-    bool resetContext = false;
-    YamiStatus status = YAMI_SUCCESS;
-
     if (m_frameHdr.key_frame != Vp8FrameHeader::KEYFRAME) {
         return YAMI_SUCCESS;
     }
@@ -83,52 +82,14 @@ YamiStatus VaapiDecoderVP8::ensureContext()
        so, m_confiBuffer width/height may update upon key frame
        3. we don't care m_configBuffer graphicBufferWidth/graphicBufferHeight for now,
        since that is used for android
-     */
+    */
 
-    DEBUG("got frame size: %d x %d", m_frameHdr.width, m_frameHdr.height);
     m_frameWidth = m_frameHdr.width;
     m_frameHeight = m_frameHdr.height;
-
-    // only reset va context when there is a larger frame
-    if (m_configBuffer.width < m_frameHdr.width
-        || m_configBuffer.height < m_frameHdr.height) {
-        resetContext = true;
-        INFO("frame size changed, reconfig codec. orig size %d x %d, new size: %d x %d", m_configBuffer.width, m_configBuffer.height, m_frameHdr.width, m_frameHdr.height);
-        m_configBuffer.width = m_frameHdr.width;
-        m_configBuffer.height = m_frameHdr.height;
-        m_configBuffer.surfaceWidth = m_configBuffer.width;
-        m_configBuffer.surfaceHeight = m_configBuffer.height;
-        if (m_hasContext)
-            status = VaapiDecoderBase::terminateVA();
-        m_hasContext = false;
-
-        if (status != YAMI_SUCCESS)
-            return status;
-    } else if (m_videoFormatInfo.width != m_frameHdr.width
-        || m_videoFormatInfo.height != m_frameHdr.height) {
-        // notify client of resolution change, no need to reset hw context
-            INFO("frame size changed, reconfig codec. orig size %d x %d, new size: %d x %d\n", m_videoFormatInfo.width, m_videoFormatInfo.height, m_frameHdr.width, m_frameHdr.height);
-            m_videoFormatInfo.width = m_frameHdr.width;
-            m_videoFormatInfo.height = m_frameHdr.height;
-            // XXX, assume graphicBufferWidth/graphicBufferHeight are hw resolution, needn't update here
-            return YAMI_DECODE_FORMAT_CHANGE;
-    }
-
-    if (m_hasContext)
-        return YAMI_SUCCESS;
-
-    DEBUG("Start VA context");
-    status = VaapiDecoderBase::start(&m_configBuffer);
-
-    if (status != YAMI_SUCCESS)
-        return status;
-
-    m_hasContext = true;
-
-    if (resetContext)
+    if (setFormat(m_frameWidth, m_frameHeight, m_frameWidth, m_frameHeight, surfaceNumVP8)) {
         return YAMI_DECODE_FORMAT_CHANGE;
-
-    return YAMI_SUCCESS;
+    }
+    return ensureProfile(VAProfileVP8Version0_3);
 }
 
 bool VaapiDecoderVP8::fillSliceParam(VASliceParameterBufferVP8* sliceParam)
@@ -314,9 +275,9 @@ bool VaapiDecoderVP8::ensureProbabilityTable(const PicturePtr&  pic)
     // XXX, create/render VAProbabilityDataBufferVP8 in base class
     if (!pic->editProbTable(probTable))
         return false;
-    memcpy (probTable->dct_coeff_probs, 
-            m_frameHdr.entropy_hdr.coeff_probs,
-            sizeof (m_frameHdr.entropy_hdr.coeff_probs));
+    memcpy(probTable->dct_coeff_probs,
+        m_frameHdr.entropy_hdr.coeff_probs,
+        sizeof(m_frameHdr.entropy_hdr.coeff_probs));
     return true;
 }
 
@@ -379,32 +340,33 @@ void VaapiDecoderVP8::updateReferencePictures()
 
 }
 
-bool VaapiDecoderVP8::allocNewPicture()
+YamiStatus VaapiDecoderVP8::allocNewPicture()
 {
-    m_currentPicture = createPicture(m_currentPTS);
+    YamiStatus status = createPicture(m_currentPicture, m_currentPTS);
 
-    if (!m_currentPicture)
-        return false;
+    if (status != YAMI_SUCCESS)
+        return status;
 
     SurfacePtr surface = m_currentPicture->getSurface();
     ASSERT(m_frameWidth && m_frameHeight);
     if (!surface->setCrop(0, 0, m_frameWidth, m_frameHeight)) {
         ASSERT(0 && "frame size is bigger than internal surface resolution");
-        return false;
+        return YAMI_FAIL;
     }
 
     DEBUG ("alloc new picture: %p with surface ID: %x",
          m_currentPicture.get(), m_currentPicture->getSurfaceID());
 
-    return true;
+    return YAMI_SUCCESS;
 }
 
 YamiStatus VaapiDecoderVP8::decodePicture()
 {
     YamiStatus status = YAMI_SUCCESS;
 
-    if (!allocNewPicture())
-        return YAMI_FAIL;
+    status = allocNewPicture();
+    if (status != YAMI_SUCCESS)
+        return status;
 
     if (!ensureQuantMatrix(m_currentPicture)) {
         ERROR("failed to reset quantizer matrix");
@@ -500,7 +462,7 @@ void VaapiDecoderVP8::stop(void)
     VaapiDecoderBase::stop();
 }
 
-void VaapiDecoderVP8::flush(void)
+void VaapiDecoderVP8::flush(bool discardOutput)
 {
     DEBUG("VP8: flush()");
     /*FIXME: should output all surfaces in drain mode*/
@@ -510,13 +472,25 @@ void VaapiDecoderVP8::flush(void)
     m_altRefPicture.reset();
     m_gotKeyFrame = false;
 
-    VaapiDecoderBase::flush();
+    if (discardOutput)
+        VaapiDecoderBase::flush();
+}
+
+void VaapiDecoderVP8::flush(void)
+{
+    DEBUG("VP8: flush()");
+    /*FIXME: should output all surfaces in drain mode*/
+    flush(true);
 }
 
 YamiStatus VaapiDecoderVP8::decode(VideoDecodeBuffer* buffer)
 {
     YamiStatus status;
     Vp8ParserResult result;
+    if (!buffer || !buffer->data) {
+        flush(false);
+        return YAMI_SUCCESS;
+    }
 
     m_currentPTS = buffer->timeStamp;
 
@@ -524,7 +498,7 @@ YamiStatus VaapiDecoderVP8::decode(VideoDecodeBuffer* buffer)
     m_frameSize = buffer->size;
 
     DEBUG("VP8: Decode(bufsize =%d, timestamp=%ld)", m_frameSize,
-          m_currentPTS);
+        m_currentPTS);
 
     do {
         if (m_frameSize == 0) {
@@ -539,6 +513,9 @@ YamiStatus VaapiDecoderVP8::decode(VideoDecodeBuffer* buffer)
             break;
         }
 
+        if (!targetTemporalFrame())
+            return YAMI_SUCCESS;
+
         if (m_frameHdr.key_frame == Vp8FrameHeader::KEYFRAME) {
             status = ensureContext();
             if (status != YAMI_SUCCESS)
@@ -590,4 +567,27 @@ YamiStatus VaapiDecoderVP8::decode(VideoDecodeBuffer* buffer)
     return status;
 }
 
+bool VaapiDecoderVP8::targetTemporalFrame()
+{
+    switch (m_configBuffer.temporalLayer) {
+    case 0: //decode all layers.
+        return true;
+    case 1:
+        if ((m_frameHdr.key_frame == Vp8FrameHeader::KEYFRAME) ||
+            m_frameHdr.refresh_last)
+            return true;
+        break;
+    case 2:
+        if ((m_frameHdr.key_frame == Vp8FrameHeader::KEYFRAME) ||
+            m_frameHdr.refresh_last || m_frameHdr.refresh_golden_frame)
+            return true;
+        break;
+    case 3:
+        return true;
+    default: //decode all layers.
+        return true;
+    }
+
+    return false;
+}
 }
