@@ -83,21 +83,6 @@ static const H264LevelLimits LevelLimits[] = {
     {51, 983040, 2},
 };
 
-#define H264_MIN_TEMPORAL_GOP 8
-#define H264_MAX_TEMPORAL_LAYER_NUM 4
-
-static uint32_t H264TempIds[H264_MAX_TEMPORAL_LAYER_NUM][H264_MIN_TEMPORAL_GOP]
-    = { { 0, 0, 0, 0, 0, 0, 0, 0 },
-        { 0, 1, 0, 1, 0, 1, 0, 1 },
-        { 0, 2, 1, 2, 0, 2, 1, 2 },
-        { 0, 3, 2, 3, 1, 3, 2, 3 } };
-
-static const float H264LayerFps[H264_MAX_TEMPORAL_LAYER_NUM] = { 7.5, 15, 30, 60 };
-
-static inline uint32_t getTemporalId(uint32_t temporalLayerNum, uint32_t frameIndex){
-     return H264TempIds[temporalLayerNum - 1][frameIndex % H264_MIN_TEMPORAL_GOP];
-}
-
 static inline bool
 _poc_greater_than (uint32_t poc1, uint32_t poc2, uint32_t max_poc)
 {
@@ -168,7 +153,6 @@ static uint8_t h264_get_profile_idc(VideoProfile profile)
 {
     uint8_t idc;
     switch (profile) {
-    case PROFILE_H264_BASELINE:
     case PROFILE_H264_CONSTRAINED_BASELINE:
         idc = 66;
         break;
@@ -242,8 +226,8 @@ bit_writer_write_trailing_bits(BitWriter *bitwriter)
 
 static BOOL
 bit_writer_write_sei(BitWriter* bitwriter,
-                     const VAEncSequenceParameterBufferH264* const seq,
-                     uint32_t temporalLayerNum)
+    const VAEncSequenceParameterBufferH264* const seq,
+    uint32_t temporalLayerNum, const LayerFrameRates& svctFrameRate)
 {
     BitWriter scalabilityInfoWriter;
     uint32_t i;
@@ -283,8 +267,9 @@ bit_writer_write_sei(BitWriter* bitwriter,
         scalabilityInfoWriter.writeBits(0, 1); // layer_output_flag[i]
 
         scalabilityInfoWriter.writeBits(0, 2); // constant_frm_bitrate_idc[i]
-        scalabilityInfoWriter.writeBits((int)floor(H264LayerFps[i] * 256 + 0.5),
-                                        16); // avg_frm_rate
+        float layerFps = (1.0 * svctFrameRate[i].frameRateNum) / svctFrameRate[i].frameRateDenom;
+        scalabilityInfoWriter.writeBits((int)floor(layerFps * 256 + 0.5),
+            16); // avg_frm_rate
 
         bit_writer_put_ue(&scalabilityInfoWriter,
                           seq->picture_width_in_mbs
@@ -342,7 +327,7 @@ bit_writer_write_sps(BitWriter* bitwriter,
     uint32_t mb_adaptive_frame_field = !seq->seq_fields.bits.frame_mbs_only_flag;
     uint32_t i = 0;
 
-    constraint_set0_flag = profile == PROFILE_H264_BASELINE;
+    constraint_set0_flag = 0;
     constraint_set1_flag = profile <= PROFILE_H264_MAIN;
     constraint_set2_flag = 0;
     constraint_set3_flag = 0;
@@ -592,11 +577,11 @@ class VaapiEncStreamHeaderH264
     typedef std::vector<uint8_t> Header;
 public:
     void setSEI(const VAEncSequenceParameterBufferH264* const seqParam,
-                uint32_t temporalLayerNum)
+        uint32_t temporalLayerNum, const LayerFrameRates& svctFrameRate)
     {
         ASSERT(m_sei.empty());
         BitWriter bs;
-        bit_writer_write_sei(&bs, seqParam, temporalLayerNum);
+        bit_writer_write_sei(&bs, seqParam, temporalLayerNum, svctFrameRate);
         bsToHeader(m_sei, bs);
     }
 
@@ -779,7 +764,6 @@ private:
         , m_poc(0)
         , m_isReference(true)
         , m_priorityId(0)
-        , m_temporalId(0)
     {
     }
 
@@ -815,18 +799,17 @@ private:
     StreamHeaderPtr m_headers;
     bool m_isReference;
     uint32_t m_priorityId;
-    uint32_t m_temporalId;
 };
 
 class VaapiEncoderH264Ref
 {
 public:
-    VaapiEncoderH264Ref(const PicturePtr& picture, const SurfacePtr& surface):
-        m_frameNum(picture->m_frameNum),
-        m_poc(picture->m_poc),
-        m_pic(surface), 
-        m_temporalId(picture->m_temporalId), 
-        m_diffPicNumMinus1(0)
+    VaapiEncoderH264Ref(const PicturePtr& picture, const SurfacePtr& surface)
+        : m_frameNum(picture->m_frameNum)
+        , m_poc(picture->m_poc)
+        , m_pic(surface)
+        , m_temporalId(picture->m_temporalID)
+        , m_diffPicNumMinus1(0)
     {
     }
     uint32_t m_frameNum;
@@ -860,7 +843,6 @@ VaapiEncoderH264::VaapiEncoderH264()
     m_videoParamAVC.enableDeblockFilter = true;
     m_videoParamAVC.deblockAlphaOffsetDiv2 = 2;
     m_videoParamAVC.deblockBetaOffsetDiv2 = 2;
-    m_videoParamAVC.temporalLayerNum = 1;
     m_videoParamAVC.priorityId = 0;
     m_videoParamAVC.enablePrefixNalUnit = false;
     m_maxOutputBuffer = H264_MIN_TEMPORAL_GOP;
@@ -919,9 +901,6 @@ void VaapiEncoderH264::checkProfileLimitation()
     VAProfile& profile = m_videoParamCommon.profile;
 
     switch (profile) {
-    case VAProfileH264Baseline:
-        // only Constrained Baseline supported right now
-        profile = VAProfileH264ConstrainedBaseline;
     case VAProfileH264ConstrainedBaseline:
         if (ipPeriod() > 1) {
             WARNING("H264 baseline profile can not support B frame encoding");
@@ -962,34 +941,6 @@ void VaapiEncoderH264::checkSvcTempLimitaion()
             = 1 << (uint32_t)ceil(log2(intraPeriod())); // make sure Gop is 2^n.
     }
 
-    // check every layer bitrate is set for BRC
-    if (bitRate()) {
-        uint32_t* layerBitRate = m_videoParamCommon.rcParams.layerBitRate;
-
-        if (layerBitRate[m_temporalLayerNum - 1] != bitRate())
-            layerBitRate[m_temporalLayerNum - 1] = bitRate();
-
-#if VA_CHECK_VERSION(0, 39, 4)
-        bool resetLayerBitRate = false;
-        uint32_t i;
-        for (i = 0; i < m_temporalLayerNum - 1; i++) {
-            if (!layerBitRate[i] || layerBitRate[i] >= layerBitRate[i + 1]) {
-                ERROR(" layer bit rate setting error, need to be reset ");
-                resetLayerBitRate = true;
-                break;
-            }
-        }
-
-        if (resetLayerBitRate) {
-            for (i = 0; i < m_temporalLayerNum - 1; i++)
-                layerBitRate[i] = bitRate()
-                                  / (1 << (m_temporalLayerNum - 1 - i));
-        }
-#else
-        if (m_isSvcT)
-            ERROR("For SVC-T BRC, please make sure libva version >= 0.39.4");
-#endif
-    }
 }
 
 void VaapiEncoderH264::resetParams ()
@@ -1011,7 +962,7 @@ void VaapiEncoderH264::resetParams ()
     DEBUG("resetParams, ensureCodedBufferSize");
     ensureCodedBufferSize();
 
-    m_temporalLayerNum = m_videoParamAVC.temporalLayerNum;
+    m_temporalLayerNum = m_videoParamCommon.temporalLayers.numLayersMinus1 + 1;
 
     // enable prefix nal unit for simulcast or svc-t
     if (m_temporalLayerNum > 1 || m_videoParamAVC.priorityId)
@@ -1019,6 +970,14 @@ void VaapiEncoderH264::resetParams ()
 
     checkProfileLimitation();
     checkSvcTempLimitaion();
+    if (m_isSvcT) {
+        VideoFrameRate frameRate;
+        frameRate.frameRateDenom = frameRateDenom();
+        frameRate.frameRateNum = frameRateNum();
+        m_temporalLayerID.reset(new AvcLayerID(frameRate, m_videoParamCommon.temporalLayerIDs, m_videoParamCommon.temporalLayers.numLayersMinus1));
+        m_temporalLayerID->getLayerFrameRates(m_svctFrameRate);
+        assert(m_temporalLayerID->getLayerNum() == (m_videoParamCommon.temporalLayers.numLayersMinus1 + 1));
+    }
 
     if (intraPeriod() == 0) {
         ERROR("intra period must larger than 0");
@@ -1063,6 +1022,15 @@ void VaapiEncoderH264::resetParams ()
 
     m_maxRefFrames =
         m_maxRefList0Count + m_maxRefList1Count;
+    if (m_isSvcT) {
+        uint32_t refFrameNum = m_temporalLayerID->getMiniRefFrameNum();
+        if (refFrameNum > m_maxOutputBuffer) {
+            ERROR("Reference frame number %d > output buffer %d", refFrameNum, m_maxOutputBuffer);
+            assert(false);
+        }
+        if (m_maxRefFrames < refFrameNum)
+            m_maxRefFrames = refFrameNum;
+    }
 
     assert((uint32_t)(1 << (m_temporalLayerNum - 1)) <= m_maxOutputBuffer);
     CLIP(m_maxRefFrames, (uint32_t)(1 << (m_temporalLayerNum - 1)), m_maxOutputBuffer);
@@ -1241,8 +1209,11 @@ YamiStatus VaapiEncoderH264::reorder(const SurfacePtr& surface, uint64_t timeSta
 
     picture->m_poc = m_frameIndex * 2;
     picture->m_priorityId = m_videoParamAVC.priorityId;
-    picture->m_temporalId = getTemporalId(m_temporalLayerNum, m_frameIndex);
-    DEBUG("m_temporalId is %d", picture->m_temporalId);
+    if (m_isSvcT)
+        picture->m_temporalID = m_temporalLayerID->getTemporalLayer(m_frameIndex);
+    else
+        picture->m_temporalID = 0;
+    DEBUG("m_temporalID is %d", picture->m_temporalID);
     m_frameIndex++;
     return YAMI_SUCCESS;
 }
@@ -1318,48 +1289,19 @@ YamiStatus VaapiEncoderH264::getCodecConfig(VideoEncOutputBuffer* outBuffer)
 void VaapiEncoderH264::fill(
     VAEncMiscParameterTemporalLayerStructure* layerParam) const
 {
+    LayerIDs ids;
+    m_temporalLayerID->getLayerIds(ids);
     layerParam->number_of_layers = m_temporalLayerNum;
-    layerParam->periodicity = H264_MIN_TEMPORAL_GOP;
+    layerParam->periodicity = ids.size();
 
     for (uint32_t i = 0; i < layerParam->periodicity; i++)
-        layerParam->layer_id[i] = getTemporalId(m_temporalLayerNum, i + 1);
+        layerParam->layer_id[i] = ids[(i + 1) % layerParam->periodicity];
 }
 #endif
 
-void VaapiEncoderH264::fill(VAEncMiscParameterRateControl* rateControl,
-                            uint32_t temporalId) const
-{
-    VaapiEncoderBase::fill(rateControl);
-
-    rateControl->bits_per_second
-        = m_videoParamCommon.rcParams.layerBitRate[temporalId];
-#if VA_CHECK_VERSION(0, 39, 4)
-    rateControl->rc_flags.bits.temporal_id = temporalId;
-#endif
-}
-
-void VaapiEncoderH264::fill(VAEncMiscParameterFrameRate* frameRate,
-                            uint32_t temporalId) const
-{
-    uint32_t expTemId = (1 << (m_temporalLayerNum - 1 - temporalId));
-    if (fps() % expTemId == 0)
-        frameRate->framerate = fps() / expTemId;
-    else
-        frameRate->framerate = (expTemId << 16 | fps());
-
-#if VA_CHECK_VERSION(0, 39, 4)
-    frameRate->framerate_flags.bits.temporal_id = temporalId;
-#endif
-}
-
 /* Generates additional control parameters */
 bool VaapiEncoderH264::ensureMiscParams(VaapiEncPicture* picture)
 {
-    VAEncMiscParameterHRD* hrd = NULL;
-    if (!picture->newMisc(VAEncMiscParameterTypeHRD, hrd))
-        return false;
-    if (hrd)
-        VaapiEncoderBase::fill(hrd);
     VideoRateControl mode = rateControlMode();
     if (mode == RATE_CONTROL_CBR || mode == RATE_CONTROL_VBR) {
 #if VA_CHECK_VERSION(0, 39, 4)
@@ -1372,21 +1314,11 @@ bool VaapiEncoderH264::ensureMiscParams(VaapiEncPicture* picture)
                 fill(layerParam);
         }
 #endif
-        for (uint32_t i = 0; i < m_temporalLayerNum; i++) {
-            VAEncMiscParameterRateControl* rateControl = NULL;
-            if (!picture->newMisc(VAEncMiscParameterTypeRateControl,
-                                  rateControl))
-                return false;
-            if (rateControl)
-                fill(rateControl, i);
-
-            VAEncMiscParameterFrameRate* frameRate = NULL;
-            if (!picture->newMisc(VAEncMiscParameterTypeFrameRate, frameRate))
-                return false;
-            if (frameRate)
-                fill(frameRate, i);
-        }
     }
+
+    if (!VaapiEncoderBase::ensureMiscParams(picture))
+        return false;
+
     return true;
 }
 
@@ -1459,7 +1391,7 @@ bool  VaapiEncoderH264::pictureReferenceListSet (
 
     for (i = 0; i < m_refList.size(); i++) {
         assert(picture->m_poc != m_refList[i]->m_poc);
-        if (picture->m_temporalId >= m_refList[i]->m_temporalId) {
+        if (picture->m_temporalID >= m_refList[i]->m_temporalId) {
             m_refList[i]->m_diffPicNumMinus1 =
                 picture->m_frameNum - m_refList[i]->m_frameNum - 1;
             if (picture->m_poc > m_refList[i]->m_poc) {
@@ -1605,7 +1537,7 @@ bool VaapiEncoderH264::ensureSequenceHeader(const PicturePtr& picture,const VAEn
 {
     m_headers.reset(new VaapiEncStreamHeaderH264());
     if (m_isSvcT)
-        m_headers->setSEI(sequence, m_temporalLayerNum);
+        m_headers->setSEI(sequence, m_temporalLayerNum, m_svctFrameRate);
     m_headers->setSPS(sequence, profile());
     return true;
 }
@@ -1657,7 +1589,7 @@ bool VaapiEncoderH264::addPackedPrefixNalUnit(const PicturePtr& picture) const
     bs.writeBits(1, 1); /* no_inter_layer_pred_flag */
     bs.writeBits(0, 3); /* dependency_id */
     bs.writeBits(0, 4); /* quality_id */
-    bs.writeBits(picture->m_temporalId, 3);
+    bs.writeBits(picture->m_temporalID, 3);
     bs.writeBits(0, 1); /* use_ref_base_pic_flag */
     bs.writeBits(1, 1); /* discardable_flag */
     bs.writeBits(1, 1); /* output_flag */
