@@ -87,6 +87,10 @@ extern "C" {
 
 #define YAMI_FOURCC_R210 YAMI_FOURCC('R', '2', '1', '0')
 
+#ifndef V4L2_BUF_FLAG_NON_REF
+#define V4L2_BUF_FLAG_NON_REF 0x00200000
+#endif
+
 typedef enum {
     NATIVE_DISPLAY_AUTO,    // decided by yami
     NATIVE_DISPLAY_X11,
@@ -108,6 +112,7 @@ typedef enum {
     VIDEO_DATA_MEMORY_TYPE_SURFACE_ID,  // it can be used for surface sharing of transcoding, benefits suppressed rendering as well.
                                         //it is discouraged to use it for video rendering.
     VIDEO_DATA_MEMORY_TYPE_ANDROID_BUFFER_HANDLE, // buffer_handle_t for android
+    VIDEO_DATA_MEMORY_TYPE_EXTERNAL_DMA_BUF, //external dma buffer, the buffer is allocate by user
 } VideoDataMemoryType;
 
 typedef struct VideoFrameRawData{
@@ -131,7 +136,6 @@ typedef enum {
     /* encoder fatal */
 
     /* decoder fatal */
-    YAMI_DECODE_NO_SURFACE,
     YAMI_DECODE_PARSER_FAIL,
 
     /* common fatal */
@@ -153,6 +157,7 @@ typedef enum {
     /* decoder no fatal */
     YAMI_DECODE_INVALID_DATA,
     YAMI_DECODE_FORMAT_CHANGE,
+    YAMI_DECODE_NO_SURFACE,
 
     /* encoder no fatal */
     YAMI_ENCODE_BUFFER_TOO_SMALL, // The buffer passed to encode is too small to contain encoded data
@@ -185,7 +190,8 @@ typedef YamiStatus Encode_Status;
 #define ENCODE_NO_MEMORY YAMI_OUT_MEMORY
 /*compatible code end*/
 
-typedef struct {
+typedef struct _SurfaceAllocParams SurfaceAllocParams;
+typedef struct _SurfaceAllocParams {
     //in
     uint32_t fourcc;
     uint32_t width;
@@ -201,6 +207,29 @@ typedef struct {
 
     //out
     intptr_t* surfaces;
+
+    /**
+     * yami will call this when it want new surface to put decode data
+     *
+     * it's optional
+     * @param user SurfaceAllocParams::user
+     * @param surface you want decode to
+     */
+    YamiStatus (*getSurface)(SurfaceAllocParams* thiz, intptr_t* surface);
+
+    /**
+     * yami will call this when all surface usage have been done
+     *
+     * it's optional
+     * @param user SurfaceAllocParams::user
+     * @param surface need to recycle
+     */
+    YamiStatus (*putSurface)(SurfaceAllocParams* thiz, intptr_t surface);
+
+    //you can set your private data for get and put surface
+    //yami will not touch it
+    void* user;
+
 } SurfaceAllocParams;
 
 typedef struct _SurfaceAllocator SurfaceAllocator;
@@ -209,7 +238,7 @@ typedef struct _SurfaceAllocator SurfaceAllocator;
 // 2. reconstruct surface for encoder.
 typedef struct _SurfaceAllocator
 {
-    void*      *user;   /* you can put your private data here, yami will not touch it */
+    void* user; /* you can put your private data here, yami will not touch it */
     /* alloc and free surfaces */
     YamiStatus (*alloc) (SurfaceAllocator* thiz, SurfaceAllocParams* params);
     YamiStatus (*free)  (SurfaceAllocator* thiz, SurfaceAllocParams* params);
