@@ -62,7 +62,6 @@ typedef enum {
     PROFILE_MPEG4_ADVANCED_SIMPLE,
     PROFILE_MPEG4_MAIN,
     PROFILE_H263_BASELINE,
-    PROFILE_H264_BASELINE,
     PROFILE_H264_CONSTRAINED_BASELINE,
     PROFILE_H264_MAIN,
     PROFILE_H264_HIGH,
@@ -87,6 +86,13 @@ typedef enum {
 #define ENCODE_BUFFERFLAG_DATAINVALID      0x00000020
 #define ENCODE_BUFFERFLAG_SLICEOVERFOLOW   0x00000040
 
+//quality level for encoder
+#define VIDEO_PARAMS_QUALITYLEVEL_NONE 0
+#define VIDEO_PARAMS_QUALITYLEVEL_MIN 1
+#define VIDEO_PARAMS_QUALITYLEVEL_MAX 7
+#define TEMPORAL_LAYER_LENGTH_MAX 32
+#define TEMPORAL_LAYERIDS_LENGTH_MAX 32
+
 typedef struct VideoEncOutputBuffer {
     uint8_t *data;
     uint32_t bufferSize;        //buffer size
@@ -94,6 +100,7 @@ typedef struct VideoEncOutputBuffer {
     uint32_t remainingSize;
     uint32_t flag;                   //Key frame, Codec Data etc
     VideoOutputFormat format;   //output format
+    uint8_t temporalID;			//output temporal id for SVCT encoder
     uint64_t timeStamp;         //reserved
 }VideoEncOutputBuffer;
 
@@ -147,6 +154,19 @@ typedef struct VideoFrameRate {
     uint32_t frameRateDenom;
 }VideoFrameRate;
 
+typedef struct VideoTemporalLayers {
+    //layer 0 bitrate is bitRate[0]
+    //...
+    //layer highest birate is VideoParamsCommon.rcParams.bitRate
+    uint8_t numLayersMinus1;
+    uint32_t bitRate[TEMPORAL_LAYER_LENGTH_MAX];
+} VideoTemporalLayers;
+
+typedef struct VideoTemporalLayerIDs {
+    uint8_t numIDs;
+    uint8_t ids[TEMPORAL_LAYERIDS_LENGTH_MAX];
+} VideoTemporalLayerIDs;
+
 typedef struct VideoResolution {
     uint32_t width;
     uint32_t height;
@@ -157,13 +177,10 @@ typedef struct VideoRateControlParams {
     uint32_t initQP;
     uint32_t minQP;
     uint32_t maxQP;
-    uint32_t windowSize; // use for HRD CPB length in ms
-    uint32_t targetPercentage;
     uint32_t disableFrameSkip;
     uint32_t disableBitsStuffing;
     int8_t diffQPIP;// P frame qp minus initQP
     int8_t diffQPIB;// B frame qp minus initQP
-    uint32_t layerBitRate[32]; // specify each scalable layer bitrate
 }VideoRateControlParams;
 
 typedef struct SliceNum {
@@ -185,6 +202,7 @@ typedef enum {
     VideoParamsTypeVP9,
     VideoParamsTypeVC1,
     VideoParamsTypeHRD,
+    VideoParamsTypeQualityLevel,
 
     VideoConfigTypeFrameRate,
     VideoConfigTypeBitRate,
@@ -197,7 +215,7 @@ typedef enum {
     VideoConfigTypeAVCStreamFormat,
 
     VideoParamsConfigExtension
-}VideoParamConfigType;
+} VideoParamConfigType;
 
 typedef struct VideoParamConfigSet {
     uint32_t size;
@@ -209,6 +227,8 @@ typedef struct VideoParamsCommon {
     uint8_t level;
     VideoResolution resolution;
     VideoFrameRate frameRate;
+    VideoTemporalLayers temporalLayers;
+    VideoTemporalLayerIDs temporalLayerIDs;
     uint32_t intraPeriod;
     uint32_t ipPeriod;
     uint32_t numRefFrames;
@@ -232,7 +252,6 @@ typedef struct VideoParamsAVC {
     bool  enableDeblockFilter;
     int8_t deblockAlphaOffsetDiv2; //same as slice_alpha_c0_offset_div2 defined in h264 spec 7.4.3
     int8_t deblockBetaOffsetDiv2; //same as slice_beta_offset_div2 defined in h264 spec 7.4.3
-    uint32_t temporalLayerNum;
     uint32_t priorityId;
     // enable prefix NAL unit defined as h264 spec G.3.42.
     // It can be used for h264 simucast or svc-t encoding.
@@ -247,8 +266,15 @@ typedef struct VideoParamsHRD {
     uint32_t size;
     uint32_t bufferSize;
     uint32_t initBufferFullness;
+    uint32_t windowSize; // use for HRD CPB length in ms
+    uint32_t targetPercentage;
 }VideoParamsHRD;
 
+typedef struct VideoParamsQualityLevel {
+    uint32_t size;
+    uint32_t level;
+} VideoParamsQualityLevel;
+
 typedef struct VideoConfigFrameRate {
     uint32_t size;
     VideoFrameRate frameRate;
