@@ -32,23 +32,12 @@
 #include "v4l2_decode.h"
 #include "common/log.h"
 #include "common/common_def.h"
+#include "vaapi/vaapidisplay.h"
 #include <algorithm>
-#ifdef ANDROID
-#include <va/va_android.h>
-#include <ufo/gralloc.h>
-#include <ufo/graphics.h>
-#elif defined(__ENABLE_WAYLAND__)
+#if defined(__ENABLE_WAYLAND__)
 #include <va/va_wayland.h>
 #endif
 
-#ifdef ANDROID
-#if !defined(EFD_SEMAPHORE)
-#define EFD_SEMAPHORE (1 << 0)
-#endif
-
-#define ANDROID_DISPLAY 0x18C34078
-#endif
-
 typedef SharedPtr < V4l2CodecBase > V4l2CodecPtr;
 #define THREAD_NAME(thread) (thread == INPUT ? "V4L2-INPUT" : "V4L2-OUTPUT")
 
@@ -83,18 +72,20 @@ typedef SharedPtr < V4l2CodecBase > V4l2CodecPtr;
 V4l2CodecBase::V4l2CodecBase()
     : m_memoryType(VIDEO_DATA_MEMORY_TYPE_RAW_COPY)
     , m_started(false)
-#ifdef ANDROID
-    , m_reqBuffCnt(0)
-#elif defined(__ENABLE_WAYLAND__) || defined(__ENABLE_X11__)
-    , m_Display(NULL)
-#else
-#endif
-    , m_drmfd(0)
+    , m_svct(false)
     , m_hasEvent(false)
     , m_inputThreadCond(m_frameLock[INPUT])
     , m_outputThreadCond(m_frameLock[OUTPUT])
     , m_eosState(EosStateNormal)
 {
+    m_maxBufferCount[INPUT] = 0;
+    m_maxBufferCount[OUTPUT] = 0;
+    m_bufferPlaneCount[INPUT] = 0;
+    m_bufferPlaneCount[OUTPUT] = 0;
+    m_memoryMode[INPUT] = 0;
+    m_memoryMode[OUTPUT] = 0;
+    m_pixelFormat[INPUT] = 0;
+    m_pixelFormat[OUTPUT] = 0;
     m_streamOn[INPUT] = false;
     m_streamOn[OUTPUT] = false;
     m_threadOn[INPUT] = false;
@@ -112,11 +103,7 @@ V4l2CodecBase::V4l2CodecBase()
     m_frameCount[OUTPUT] = 0;
 #endif
 
-#ifdef ANDROID
-    // it's better done inside vaapi driver (retrieve buffer picth etc)
-    hw_get_module(GRALLOC_HARDWARE_MODULE_ID, (hw_module_t const**)&m_pGralloc);
-    ASSERT(m_pGralloc);
-#endif
+    memset(&m_nativeDisplay, 0, sizeof(m_nativeDisplay));
 }
 
 V4l2CodecBase::~V4l2CodecBase()
@@ -461,16 +448,6 @@ int32_t V4l2CodecBase::ioctl(int command, void* arg)
             GET_PORT_INDEX(port, dqbuf->type, ret);
             ASSERT(ret != -1);
             ASSERT(dqbuf->memory == m_memoryMode[port]);
-            if (port == OUTPUT) {
-            #ifdef ANDROID
-                if (m_streamOn[port] == false) {
-                    // simple deque the unsed output buffer for android surface's cancelBuffer()
-                    dqbuf->index = m_framesTodo[port].front();
-                    m_framesTodo[port].pop_front();
-                    break;
-                }
-            #endif
-            }
 
             DEBUG("m_framesDone[%d].size(): %zu, m_reqBufState[port]: %d",
                 port, m_framesDone[port].size(), m_reqBufState[port]);
@@ -574,7 +551,10 @@ int32_t V4l2CodecBase::poll(bool poll_device, bool* event_pending)
       return -1;
     }
 
-    *event_pending = m_hasEvent;
+    {
+        YamiMediaCodec::AutoLock locker(m_codecLock);
+        *event_pending = m_hasEvent;
+    }
 
     // clear event
     if (pollfds[1].revents & POLLIN)
@@ -583,10 +563,20 @@ int32_t V4l2CodecBase::poll(bool poll_device, bool* event_pending)
     return 0;
 }
 
+bool V4l2CodecBase::hasCodecEvent()
+{
+        YamiMediaCodec::AutoLock locker(m_codecLock);
+        return m_hasEvent;
+}
+
 void V4l2CodecBase::setCodecEvent()
 {
-    YamiMediaCodec::AutoLock locker(m_codecLock);
-    m_hasEvent = true;
+    {
+        YamiMediaCodec::AutoLock locker(m_codecLock);
+        m_hasEvent = true;
+    }
+    //notify user about this
+    setDeviceEvent(0);
 }
 
 void V4l2CodecBase::clearCodecEvent()
@@ -600,14 +590,6 @@ struct FormatEntry {
     const char* mime;
 };
 
-#ifndef V4L2_PIX_FMT_VP9
-#define V4L2_PIX_FMT_VP9 YAMI_FOURCC('V', 'P', '9', '0')
-#endif
-
-#ifndef V4L2_PIX_FMT_VC1
-#define V4L2_PIX_FMT_VC1 YAMI_FOURCC('V', 'C', '1', '0')
-#endif
-
 static const FormatEntry FormatEntrys[] = {
     {V4L2_PIX_FMT_H264, YAMI_MIME_H264},
     {V4L2_PIX_FMT_VP8, YAMI_MIME_VP8},
@@ -644,171 +626,30 @@ const char* mimeFromV4l2PixelFormat(uint32_t pixelFormat)
     return mime;
 }
 
-class VideoFrameDeleter {
-public:
-    VideoFrameDeleter(VADisplay display)
-        : m_display(display)
-    {
-    }
-    void operator()(VideoFrame* frame)
-    {
-        VASurfaceID id = (VASurfaceID)frame->surface;
-        DEBUG("destroy VASurface 0x%x", id);
-        vaDestroySurfaces(m_display, &id, 1);
-        delete frame;
-    }
+#if defined(__ENABLE_WAYLAND__)
+bool V4l2CodecBase::setWaylandDisplay(struct wl_display* wlDisplay)
+{
+    m_nativeDisplay.type = NATIVE_DISPLAY_WAYLAND;
+    m_nativeDisplay.handle = (intptr_t)wlDisplay;
+    return true;
+};
+#else
 
-private:
-    VADisplay m_display;
+#if defined(__ENABLE_X11__)
+bool V4l2CodecBase::setXDisplay(Display* x11Display)
+{
+    m_nativeDisplay.type = NATIVE_DISPLAY_X11;
+    m_nativeDisplay.handle = (intptr_t)x11Display;
+    return true;
+};
+#endif //__ENABLE_X11__
+
+#endif //__ENABLE_WAYLAND__
+
+bool V4l2CodecBase::setDrmFd(int fd)
+{
+    m_nativeDisplay.type = NATIVE_DISPLAY_DRM;
+    m_nativeDisplay.handle = (intptr_t)fd;
+    return true;
 };
 
-#ifdef ANDROID
-bool V4l2CodecBase::setVaDisplay()
-{
-    unsigned int display = ANDROID_DISPLAY;
-    m_vaDisplay = vaGetDisplay(&display);
-
-    int major, minor;
-    VAStatus status;
-    status = vaInitialize(m_vaDisplay, &major, &minor);
-    if (status != VA_STATUS_SUCCESS) {
-        fprintf(stderr, "init vaDisplay failed\n");
-        return false;
-    }
-
-    return true;
-}
-
-bool V4l2CodecBase::createVpp()
-{
-    NativeDisplay nativeDisplay;
-    nativeDisplay.type = NATIVE_DISPLAY_VA;
-    nativeDisplay.handle = (intptr_t)m_vaDisplay;
-    m_vpp.reset(createVideoPostProcess(YAMI_VPP_SCALER), releaseVideoPostProcess);
-    return m_vpp->setNativeDisplay(nativeDisplay) == YAMI_SUCCESS;
-}
-SharedPtr<VideoFrame> V4l2CodecBase::createVaSurface(const buffer_handle_t buf_handle, int32_t width, int32_t height)
-{
-    SharedPtr<VideoFrame> frame;
-
-    intel_ufo_buffer_details_t info;
-    memset(&info, 0, sizeof(info));
-    *reinterpret_cast<uint32_t*>(&info) = sizeof(info);
-
-    int err = 0;
-    // it's better skip pitch attribute here, but vaapi driver retrieve the pitch by itself
-    if (m_pGralloc)
-        err = m_pGralloc->perform(m_pGralloc, INTEL_UFO_GRALLOC_MODULE_PERFORM_GET_BO_INFO, buf_handle, &info);
-
-    if (0 != err || !m_pGralloc) {
-        fprintf(stderr, "create vaSurface failed\n");
-        return frame;
-    }
-
-    VASurfaceAttrib attrib;
-    memset(&attrib, 0, sizeof(attrib));
-
-    VASurfaceAttribExternalBuffers external;
-    memset(&external, 0, sizeof(external));
-
-    external.pixel_format = VA_FOURCC_NV12;
-    external.width = width;
-    external.height = height;
-    external.pitches[0] = info.pitch;
-    external.num_planes = 2;
-    external.num_buffers = 1;
-    uint8_t* handle = (uint8_t*)buf_handle;
-    external.buffers = (long unsigned int*)&handle; //graphic handel
-    external.flags = VA_SURFACE_ATTRIB_MEM_TYPE_ANDROID_GRALLOC;
-
-    attrib.flags = VA_SURFACE_ATTRIB_SETTABLE;
-    attrib.type = (VASurfaceAttribType)VASurfaceAttribExternalBufferDescriptor;
-    attrib.value.type = VAGenericValueTypePointer;
-    attrib.value.value.p = &external;
-
-    VASurfaceID id;
-    VAStatus vaStatus = vaCreateSurfaces(m_vaDisplay, VA_RT_FORMAT_YUV420,
-        width, height, &id, 1, &attrib, 1);
-    if (vaStatus != VA_STATUS_SUCCESS)
-        return frame;
-
-    frame.reset(new VideoFrame, VideoFrameDeleter(m_vaDisplay));
-    memset(frame.get(), 0, sizeof(VideoFrame));
-
-    frame->surface = static_cast<intptr_t>(id);
-    frame->crop.width = width;
-    frame->crop.height = height;
-
-    return frame;
-}
-
-bool V4l2CodecBase::mapVideoFrames(int32_t width, int32_t height)
-{
-    SharedPtr<VideoFrame> frame;
-
-    DEBUG("create_surface: %d x %d\n", width, height);
-    for (uint32_t i = 0; i < m_bufferHandle.size(); i++) {
-        frame = createVaSurface(m_bufferHandle[i], width, height);
-        if (!frame)
-            return false;
-        m_videoFrames.push_back(frame);
-    }
-    return true;
-}
-#elif defined(__ENABLE_WAYLAND__)
-bool V4l2CodecBase::setVaDisplay()
-{
-    VAStatus status;
-    int major, minor;
-    m_vaDisplay = vaGetDisplayWl((struct wl_display*)m_Display);
-    status = vaInitialize(m_vaDisplay, &major, &minor);
-    if (status != VA_STATUS_SUCCESS)
-        return false;
-    return true;
-}
-
-bool V4l2CodecBase::createVpp()
-{
-    NativeDisplay nativeDisplay;
-    nativeDisplay.type = NATIVE_DISPLAY_VA;
-    nativeDisplay.handle = (intptr_t)m_vaDisplay;
-    m_vpp.reset(createVideoPostProcess(YAMI_VPP_SCALER), releaseVideoPostProcess);
-    return m_vpp->setNativeDisplay(nativeDisplay) == YAMI_SUCCESS;
-}
-
-SharedPtr<VideoFrame> V4l2CodecBase::createVaSurface(uint32_t width, uint32_t height)
-{
-    VASurfaceID id;
-    VASurfaceAttrib attrib;
-    SharedPtr<VideoFrame> frame;
-    uint32_t rtFormat = VA_RT_FORMAT_YUV420;
-    int pixelFormat = VA_FOURCC_NV12;
-    attrib.type = VASurfaceAttribPixelFormat;
-    attrib.flags = VA_SURFACE_ATTRIB_SETTABLE;
-    attrib.value.type = VAGenericValueTypeInteger;
-    attrib.value.value.i = pixelFormat;
-
-    VAStatus vaStatus = vaCreateSurfaces(m_vaDisplay, rtFormat, width, height, &id, 1, &attrib, 1);
-    if (vaStatus != VA_STATUS_SUCCESS)
-        return frame;
-
-    frame.reset(new VideoFrame, VideoFrameDeleter(m_vaDisplay));
-    memset(frame.get(), 0, sizeof(VideoFrame));
-    frame->surface = static_cast<intptr_t>(id);
-    frame->crop.width = width;
-    frame->crop.height = height;
-    return frame;
-}
-
-bool V4l2CodecBase::mapVideoFrames(uint32_t width, uint32_t height)
-{
-    SharedPtr<VideoFrame> frame;
-    for (uint32_t i = 0; i < m_reqBuffCnt; i++) {
-        frame = createVaSurface(width, height);
-        if (!frame)
-            return false;
-        m_videoFrames.push_back(frame);
-    }
-    return true;
-}
-#endif
