@@ -107,6 +107,13 @@ int32_t YamiV4L2_FrameMemoryType(int32_t fd, VideoDataMemoryType memory_type)
     return v4l2Codec->setFrameMemoryType(memory_type);
 }
 
+int32_t YamiV4L2_SvcT(int32_t fd, bool enable)
+{
+    V4l2CodecPtr v4l2Codec = _findCodecFromFd(fd);
+    ASSERT(v4l2Codec);
+    return v4l2Codec->setSvcT(enable);
+}
+
 int32_t YamiV4L2_Ioctl(int32_t fd, int command, void* arg)
 {
     V4l2CodecPtr v4l2Codec = _findCodecFromFd(fd);
@@ -148,8 +155,7 @@ int32_t YamiV4L2_Munmap(void* addr, size_t length)
     return 0;
 }
 
-#ifdef  ANDROID
-#elif defined(__ENABLE_WAYLAND__)
+#if defined(__ENABLE_WAYLAND__)
 int32_t YamiV4L2_SetWaylandDisplay(int32_t fd, struct wl_display* wlDisplay)
 {
     V4l2CodecPtr v4l2Codec = _findCodecFromFd(fd);
@@ -222,6 +228,9 @@ int32_t YamiV4L2_SetParameter(int32_t fd, const char* key, const char* value)
             memoryType = VIDEO_DATA_MEMORY_TYPE_SURFACE_ID;
         } else if (!strcmp(value, "android-buffer-handle")) {
             memoryType = VIDEO_DATA_MEMORY_TYPE_ANDROID_BUFFER_HANDLE;
+        }
+        else if (!strcmp(value, "external-dma-buf")) {
+            memoryType = VIDEO_DATA_MEMORY_TYPE_EXTERNAL_DMA_BUF;
         } else {
             ERROR("unknow output frame memory type: %s\n", value);
             return -1;
@@ -240,7 +249,11 @@ int32_t YamiV4L2_SetParameter(int32_t fd, const char* key, const char* value)
         Display* x11Display = (Display*)ptr;
         DEBUG("x11Display: %p", x11Display);
         ret = v4l2Codec->setXDisplay(x11Display);
-    #endif
+#endif
+    } else if(!(strcmp(key, "encode-mode"))) {
+        if (!strcmp(value, "svct")) {
+            ret = v4l2Codec->setSvcT(true);
+        }
     } else {
         ERROR("unsupported parameter key: %s\n", key);
     }
