@@ -107,8 +107,9 @@ VaapiPostProcessScaler::process(const SharedPtr<VideoFrame>& src,
     SurfacePtr surface(new VaapiSurface(dest));
     VaapiVppPicture picture(m_context, surface);
     VAProcPipelineParameterBuffer* vppParam;
-    if (!picture.editVppParam(vppParam))
+    if (!picture.editVppParam(vppParam)) {
         return YAMI_OUT_MEMORY;
+    }
     VARectangle srcCrop, destCrop;
     if (fillRect(srcCrop, src->crop))
         vppParam->surface_region = &srcCrop;
@@ -322,10 +323,21 @@ YamiStatus VaapiPostProcessScaler::setColorBalanceParam(const VPPColorBalancePar
             if (mapToVppColorBalanceMode(vppClrBalanceMode, caps[i].type)) {
                 m_colorBalance[vppClrBalanceMode].range = caps[i].range;
                 m_colorBalance[vppClrBalanceMode].type = caps[i].type;
-                m_colorBalance[vppClrBalanceMode].level = COLORBALANCE_NONE;
+                m_colorBalance[vppClrBalanceMode].level = COLORBALANCE_LEVEL_NONE;
             }
         }
     }
+
+    if(COLORBALANCE_NONE == colorbalance.mode){
+        for (ColorBalanceMapItr itr = m_colorBalance.begin(); itr != m_colorBalance.end(); itr++) {
+            if (itr->second.filter) {
+                itr->second.filter.reset();
+                itr->second.level = COLORBALANCE_LEVEL_NONE;
+            }
+        }
+        return YAMI_SUCCESS;
+    }
+
     ColorBalanceMapItr iteratorClrBalance = m_colorBalance.find(colorbalance.mode);
     if (iteratorClrBalance == m_colorBalance.end()) {
         ERROR("unsupported VppColorBalanceMode: %d", colorbalance.mode);
